// EQ5 motor drive controller alternative firmware.
// Authors: LifeIsGood, Yvk
// Minor fix: mpyat2 (20170824): StopDEC() function. 

#include <AT89X51.H>
#include <EQ5Firm.h>

#define TC			9	/* Timer correction */

code unsigned char	HalfStep[8] = { 0x99, 0xB1, 0xA5, 0xE4, 0x66, 0x72, 0x5A, 0xD8 };
code unsigned char	FullStep[8] = { 0x99, 0xA5, 0x66, 0x5A, 0x99, 0xA5, 0x66, 0x5A };

unsigned int	ReloadRA, ReloadDEC;
unsigned char	TempP0, TempP2;

bit				FullStepRA, FullStepDEC;
bit				RaPosDir, DecPosDir;
unsigned char	StepRA, StepDEC;

unsigned char	RAPosition = 0, DECPosition = 0;

unsigned char	CurrentSpeed;

void DoSteps(void);

/*
 * Reload values for timers ;
 * Star speed
 */
unsigned int	ReloadRADefault = 0xC37B + TC;

/*
 * RA reload values for N - Visual ;
 * +2x, +8x, +16x, -2x, -8x, -16x ;
 * code unsigned int ReloadRA_V[6]={0xE1BD+TC, 0xF3E5+TC, 0xF947+TC, 0, 0xEBD4+TC, 0xF75B+TC};
 */
code unsigned int	ReloadRA_V[6] = { 0xE1BD + TC, 0xF3E5 + TC, 0xFA80 + TC, 0, 0xEBD4 + TC, 0xF947 + TC };

// +1x, +4x, +21x, -1x, -4x, -19x

/*
 * RA reload values for S - Astrophoto ;
 * +0.3x, +0.7x, +1x, -0.3x, -0.7x, -1x,
 */
code unsigned int	ReloadRA_P[6] = { 0xD172 + TC, 0xDC66 + TC, 0xE1BD + TC, 0xA98B + TC, 0x3644 + TC, 0 };

/*
 * DEC reload values for N - Visual ;
 * +2x, +8x, +16x, -2x, -8x, -16x
 */
code unsigned int	ReloadDEC_V[6] = { 0xC37B + TC, 0xF0DF + TC, 0xF9F3 + TC, 0xC37B + TC, 0xF0DF + TC, 0xF9F3 + TC };

/*
 * DEC reload values for S - Astrophoto ;
 * +0.3x, +0.7x, +1x, -0.3x, -0.7x, -1x,
 */
code unsigned int	ReloadDEC_P[6] = { 0x3644 + TC, 0xA98B + TC, 0xC37B + TC, 0x3644 + TC, 0xA98B + TC, 0xC37B + TC };

// [mpyat2] addtional variables.
bit localButtonDECPlus;
bit localButtonDECMinus;
bit localButtonRAPlus;
bit localButtonRAMinus;
bit localToggleSpeed4x;
bit localToggleSpeed2x;

void main(void)
{
	// [mpyat2] the next two lines are meaningless; the variables are set to zeros immediately after.
	//ReloadRA = ReloadRADefault;
	//ReloadDEC = ReloadRADefault;

	ReloadRA = 0;
	ReloadDEC = 0;

	// Inputs (setting 1 makes this pin input)
	ButtonDECPlus = 1;
	ButtonDECMinus = 1;
	ButtonRAPlus = 1;
	ButtonRAMinus = 1;
	Hemisphere = 1;
	ToggleSpeed4x = 1;
	ToggleSpeed2x = 1;

	// We are working with TempP0, TempP2 and then write them to ports.
	TempP0 = 0x00;
	TempP2 = 0xFF;
	P0 = TempP0;
	P2 = TempP2;

	InitTimers();
	ReloadRA = ReloadRADefault;

	while (1)		/* Main cycle. Check buttons do corrections */
	{
		DoSteps();

// [mpyat2] local vars: it's better to get all input values at the beginning of the cycle
// to avoid possible jittering of buttons.
        localButtonDECPlus = ButtonDECPlus;
        localButtonDECMinus = ButtonDECMinus;
        localButtonRAPlus = ButtonRAPlus;
        localButtonRAMinus = ButtonRAMinus;
        localToggleSpeed4x = ToggleSpeed4x;
        localToggleSpeed2x = ToggleSpeed2x;
// 

		/* Set Reload for timers. */
		CurrentSpeed = localToggleSpeed4x == 0 ? 1 : (localToggleSpeed2x == 0 ? 0 : 2);

		if (CurrentSpeed == 2 && Hemisphere == Visual)
			FullStepDEC = 1;
		else
			FullStepDEC = 0;
		if ((CurrentSpeed == 2 && Hemisphere == Visual) && (!localButtonRAMinus || !localButtonRAPlus))
			FullStepRA = 1;
		else
			FullStepRA = 0;
		//	if(ToggleSpeed4x==0)CurrentSpeed=1;
		//	if(ToggleSpeed2x==0)CurrentSpeed=0;
		//	if(ToggleSpeed4x==1 && ToggleSpeed2x==1)
		//		CurrentSpeed=2;
		if (localButtonRAPlus == 1 && localButtonRAMinus == 1)
		{
			ReloadRA = ReloadRADefault;
		}

		if (localButtonDECPlus == 1 && localButtonDECMinus == 1)
		{
			ET1 = 0;
			StopDEC();
			StepDEC = 0; // just in case.
		}

		/* North - Visual */
		if (Hemisphere == Visual)
		{
			/* Set delays for visual mode */
			if (localButtonRAMinus == 0)
			{
				ReloadRA = ReloadRA_V[CurrentSpeed];
			}

			if (localButtonRAPlus == 0)
			{
				ReloadRA = ReloadRA_V[3 + CurrentSpeed];
			}

			if (localButtonDECMinus == 0)
			{
				ReloadDEC = ReloadDEC_V[CurrentSpeed];
				ET1 = 1;
			}

			if (localButtonDECPlus == 0)
			{
				ReloadDEC = ReloadDEC_V[3 + CurrentSpeed];
				ET1 = 1;
			}
		}
		else		/* South - Astrophoto. No reverse for RA. Only make it slower. */
		{
			/* Set delays for astrophoto mode */
			if (localButtonRAMinus == 0)
			{
				ReloadRA = ReloadRA_P[CurrentSpeed];
			}

			if (localButtonRAPlus == 0)
			{
				ReloadRA = ReloadRA_P[3 + CurrentSpeed];
			}

			if (localButtonDECMinus == 0)
			{
				ReloadDEC = ReloadDEC_P[CurrentSpeed];
				ET1 = 1;
			}

			if (localButtonDECPlus == 0)
			{
				ReloadDEC = ReloadDEC_P[3 + CurrentSpeed];
				ET1 = 1;
			}
		}

		if (localButtonDECPlus == 0)
			DecPosDir = Decrease;
		if (localButtonDECMinus == 0)
			DecPosDir = Increase;

		RaPosDir = Decrease;

		if (localButtonRAMinus == 0 || Hemisphere == Astrophoto)
			RaPosDir = Decrease;
		if (localButtonRAPlus == 0 && Hemisphere == Visual)
			RaPosDir = Increase;

		if (localButtonRAPlus == 0 || localButtonRAMinus == 0)
			LedRed = 1;
		else
			LedRed = 0;

		if (localButtonDECPlus == 0 || localButtonDECMinus == 0)
			LedGreen = 1;
		else
			LedGreen = 0;
	}
}

void StopDEC(void)
{
	// [mpyat2] it seems we do not need such complicate expressions (the next two lines)
	//P0 = (P0 & 0xF0) | (FullStep[8] & 0x0F);
	//P2 = (P2 & 0x0F) | (FullStep[8] & 0xF0);

	// [mpyat2]: main fix. 
	// In the previous version values were set to P0/P2 ports directly.
	// However, the program relays on the values of TempP0/TempP2, 
	// so they became unsynchronized with P0/P2 values after assignment (see above).
	// First we need to assign values to TempP0/TempP2 (like in DoSteps()) 
	//TempP0 = (P0 & 0xF0);
	//TempP2 = (P2 | 0xF0);
	//P0 = TempP0;
	//P2 = TempP2;

	//[mpyat2]: additional tiny change to make even more consistence with other code (avoid to read P0/P2)
	TempP0 = (TempP0 & 0xF0);
	TempP2 = (TempP2 | 0xF0);
	P0 = TempP0;
	P2 = TempP2;
}

void DoSteps(void)
{
	unsigned char	RAPhase, DECPhase;

	if (StepDEC != 0 && ReloadDEC != 0)
	{
		StepDEC = 0;
		DECPhase = FullStepDEC == 1 ? FullStep[DECPosition] : HalfStep[DECPosition];

		/*
		 * DECPhase=FullStep[DECPosition];
		 */
		TempP0 = TempP0 & (0xF0 | DECPhase);
		TempP2 = TempP2 | (0xF0 & DECPhase);
		P0 = TempP0;
		P2 = TempP2;
		TempP0 = (TempP0 & 0xF0) | (DECPhase & 0x0F);
		TempP2 = (TempP2 & 0x0F) | (DECPhase & 0xF0);
		P0 = TempP0;
		P2 = TempP2;

		if (DecPosDir)
			DECPosition = ++DECPosition & 7;
		else
			DECPosition = --DECPosition & 7;
	}

	if (StepRA != 0 & ReloadRA != 0)
	{
		StepRA = 0;
		RAPhase = FullStepRA == 1 ? FullStep[RAPosition] : HalfStep[RAPosition];

		// RAPhase=HalfStep[RAPosition];

		RAPhase = (RAPhase << 4) | (RAPhase >> 4);
		TempP0 = TempP0 & (0x0F | RAPhase);
		TempP2 = TempP2 | (0x0F & RAPhase);
		P0 = TempP0;
		P2 = TempP2;
		TempP0 = (TempP0 & 0x0F) | (RAPhase & 0xF0);
		TempP2 = (TempP2 & 0xF0) | (RAPhase & 0x0F);
		P0 = TempP0;
		P2 = TempP2;
		if (RaPosDir)
			RAPosition = ++RAPosition & 7;
		else
			RAPosition = --RAPosition & 7;
	}
}

void InitTimers(void)
{
	TR0 = 0;
	TMOD |= 0x01;
	TL0 = 0x77;
	TH0 = 0x77;
	ET0 = 1;
	TR0 = 1;

	TR1 = 0;
	TMOD |= 0x10;
	TL1 = 0xAA;
	TH1 = 0xAA;
	ET1 = 1;
	TR1 = 1;
	EA = 1;
}

timer0_int()
interrupt TF0_VECTOR using 1
{
	unsigned int	Reload;

	Reload = ReloadRA;

	Reload += TL0;
	TL0 = Reload & 0xFF;
	TH0 += Reload >> 8;
	ET0 = 1;
	EA = 1;
	TR0 = 1;
	StepRA = 1;
}

timer1_int()
interrupt TF1_VECTOR using 2
{
	unsigned int	Reload;

	Reload = ReloadDEC;

	Reload += TL1;
	TL1 = Reload & 0xFF;
	TH1 += Reload >> 8;
	ET1 = 1;
	EA = 1;
	TR1 = 1;
	StepDEC = 1;
}